#ifdef __cplusplus
extern "C" {
#endif





	// NightFox LIB - Funciones de Colisiones
	// Requiere DevkitARM R28
	// Codigo por NightFox
	// http://blogs.gamefilia.com/knightfox
	// Version 20100301



	// Includes devKitPro
	#include <nds.h>
	#include <filesystem.h>
	#include <fat.h>

	// Includes C++
	#include <stdio.h>
	#include <string.h>
	#include <unistd.h>

	// Includes propios
	#include "nf_basic.h"
	#include "nf_colision.h"




	// Define los buffers y estructuras de control de los mapas de colision
	NF_TYPE_CMAP_INFO NF_CMAP[NF_SLOTS_CMAP];



	// Funcion NF_InitCmapBuffers();
	void NF_InitCmapBuffers(void) {
		u8 n = 0;
		for (n = 0; n < NF_SLOTS_CMAP; n ++) {
			NF_CMAP[n].buffer = NULL;	// Inicializa los punteros de los buffers
			NF_CMAP[n].size = 0;		// Tamao de archivo
			NF_CMAP[n].width = 0;		// Ancho del mapa
			NF_CMAP[n].height = 0;		// Alto del mapa
			NF_CMAP[n].inuse = false;	// Esta en uso el slot?
		}
	}



	// Funcion NF_ResetCmapBuffers();
	void NF_ResetCmapBuffers(void) {
		u8 n = 0;
		for (n = 0; n < NF_SLOTS_CMAP; n ++) {
			free(NF_CMAP[n].buffer);	// Vacia los buffers
		}
		NF_InitCmapBuffers();			// Y reinicia todas las variables
	}



	// Funcion NF_LoadColisionMap();
	void NF_LoadColisionMap(const char* file, u8 id, u16 width, u16 height) {

		// Verifica el rango de Id's
		if ((id < 0) || (id >= NF_SLOTS_CMAP)) {
			NF_Error(106, "Colision Map", NF_SLOTS_CMAP);
		}

		// Verifica si la Id esta libre
		if (NF_CMAP[id].inuse) {
			NF_Error(109, "Colision Map", id);
		}

		// Vacia los buffers que se usaran
		free(NF_CMAP[id].buffer);
		NF_CMAP[id].buffer = NULL;

		// Declara los punteros a los ficheros
		FILE* file_id;

		// Variable para almacenar el path al archivo
		char filename[256];
		
		// Carga el archivo .CMP
		sprintf(filename, "%s/%s.cmp", NF_ROOTFOLDER, file);
		file_id = fopen(filename, "rb");
		if (file_id) {	// Si el archivo existe...
			// Obten el tamao del archivo
			fseek(file_id, 0, SEEK_END);
			NF_CMAP[id].size = ftell(file_id);
			rewind(file_id);
			// Reserva el espacio en RAM
			NF_CMAP[id].buffer = (char*) calloc (NF_CMAP[id].size, sizeof(char));
			if (NF_CMAP[id].buffer == NULL) {		// Si no hay suficiente RAM libre
				NF_Error(102, NULL, NF_CMAP[id].size);
			}
			// Lee el archivo y ponlo en la RAM
			fread(NF_CMAP[id].buffer, 1, NF_CMAP[id].size, file_id);
		} else {	// Si el archivo no existe...
			NF_Error(101, filename, 0);
		}
		fclose(file_id);		// Cierra el archivo
		// swiWaitForVBlank();		// Espera al cierre del archivo (Usar en caso de corrupcion de datos)

		// Guarda las medidas
		NF_CMAP[id].width = width;
		NF_CMAP[id].height = height;

		// Y marca esta ID como usada
		NF_CMAP[id].inuse = true;

	}



	// Funcion NF_UnloadColisionMap();
	void NF_UnloadColisionMap(u8 id) {

		// Verifica el rango de Id's
		if ((id < 0) || (id >= NF_SLOTS_CMAP)) {
			NF_Error(106, "Colision Map", NF_SLOTS_CMAP);
		}

		// Verifica si la Id esta libre
		if (!NF_CMAP[id].inuse) {
			NF_Error(110, "Colision Map", id);
		}

		// Vacia los buffers que se usaran
		free(NF_CMAP[id].buffer);
		NF_CMAP[id].buffer = NULL;

		// Y marca esta ID como usada
		NF_CMAP[id].inuse = false;

	}



	// Funcion NF_GetTile();
	u8 NF_GetTile(u8 slot, u16 x, u16 y) {

		// Calcula el ancho en tiles del mapa
		u16 columns = (NF_CMAP[slot].width >> 3);		// (width / 8);

		// Calcula los tiles de posicion	(x / 8); (y / 8);
		u16 tile_x = (x >> 3);
		u16 tile_y = (y >> 3) + 1;		// +1, por que la primera fila se reserva para la referencia de tiles

		// Calcula el n de tile
		u16 tile = (((tile_y * columns) + tile_x) << 1);
		// n de tile x2, dado que el mapa es de 16 bits (2 bytes por dato) y el buffer
		// es de 8 bits, se lee el 2do byte, por eso se multiplica por 2.

		return NF_CMAP[slot].buffer[tile];

	}



	// Funcion NF_SetTile();
	void NF_SetTile(u8 slot, u16 x, u16 y, u8 value) {

		// Calcula el ancho en tiles del mapa
		u16 columns = (NF_CMAP[slot].width >> 3);		// (width / 8);

		// Calcula los tiles de posicion	(x / 8); (y / 8);
		u16 tile_x = (x >> 3);
		u16 tile_y = (y >> 3) + 1;		// +1, por que la primera fila se reserva para la referencia de tiles

		// Calcula el n de tile
		u16 tile = (((tile_y * columns) + tile_x) << 1);
		// n de tile x2, dado que el mapa es de 16 bits (2 bytes por dato) y el buffer
		// es de 8 bits, se lee el 2do byte, por eso se multiplica por 2.

		// Escribe el nuevo valor en el mapa de colisiones
		NF_CMAP[slot].buffer[tile] = value;

	}




#ifdef __cplusplus
}
#endif