struct Fondo{
	//Ruta en la que esta guardada la informacion del fondo (incluye nombre del fichero)
	char ruta[128];
	//Nombre con el que se identificara el fondo
	char nombre[128];
	//Alto del fondo sin incluir tramos de 8 pixeles de alto para definicion del "tileset"
	//Se define un tileset de esta forma cuando queremos saber que identificador se asigna a cada "tile"
	int alto;
	//Ancho del fondo
	int ancho;
	//Cantidad de pixeles verticales (multiplo de 8) que estan siendo usadas para definir los "tiles" del "tileset"
	int defTiles;
	//Margen horizontal en pixeles (multiplo de 8) que se ha aadido al fondo para que un personaje de 32x64 cuadre
	//con las dos casillas centrales en el mapa de colision.
	int margenX;
	Pantalla pantalla;
	Capa capa;
	int posX, posY;
	
	//Carga el fondo y sus datos (tileset) a la RAM
	void cargarRAM();
	//Borra el fondo y su tileset de la RAM
	void borrarRAM();
	//Carga los datos del fondo a la VRAM
	void cargarVRAM();
	//Borra los datos del fonfo de la VRAM
	void borrarVRAM();
	//Muestra el fondo
	void mostrar();
	//Oculta el fondo
	void ocultar();
	//Mueve el fondo a la posicion indicada (instantaneamente)
	void mover(int x, int y);
	//Mueve el fondo a la direccion indicada a la velocidad indicada en pixeles
	bool mover(Direccion d,int vel);
	//Pone el fondo de forma que el Sprite pasado este en el "centro"
	void centrar(Sprite s);

	//NF_TILEDBG_LAYERS
};


void Fondo::cargarRAM(){
	NF_LoadTiledBg(ruta, nombre, ancho, alto);
}
void Fondo::borrarRAM(){
	NF_UnloadTiledBg(nombre);
}
void Fondo::cargarVRAM(){
	NF_CreateTiledBg(pantalla, capa, nombre);
	NF_ScrollBg(pantalla, capa, posX, posY);
}
void Fondo::borrarVRAM(){
	NF_DeleteTiledBg(pantalla, capa);
}
void Fondo::mostrar(){
	NF_ShowBg(pantalla, capa);
}
void Fondo::ocultar(){
	NF_HideBg(pantalla, capa);
}
void Fondo::mover(int x, int y){
	posX = x;
	posY = y + defTiles;
	NF_ScrollBg(pantalla, capa, posX, posY);
}
bool Fondo::mover(Direccion d, int vel){
	bool borde = false;
	if (d == abajo){
		posY += vel;
		if (posY >= alto - 192 + defTiles){
			posY = alto - 192 + defTiles;		// Limite fisico de la capa
			borde = true;
		}
	}else if (d == arriba){
		posY -= vel;
		if (posY <= defTiles){
			posY = defTiles;		// Limite fisico de la capa
			borde = true;
		}
	}else if (d == derecha){
		posX += vel;
		if (posX >= ancho - 256 - margenX){
			posX = ancho - 256 - margenX;		// Limite fisico de la capa
			borde = true;
		}
	}else{
		posX -= vel;
		if (posX <= margenX){
			posX = margenX;		// Limite fisico de la capa
			borde = true;
		}
	}
	
	NF_ScrollBg(pantalla, capa, posX, posY);
	return borde;
	
}

void Fondo::centrar(Sprite s){
	this->mover(s.posXMapa - 112, s.posYMapa - 96);
}