

u8 estaColor(u8 screen, u8 layer, u8 r, u8 g, u8 b){
	u8 ret = 255;
	for(u8 i=0;(i<coloresFuenteUsados[screen][layer]) && (ret == 255);i++){
		if (coloresFuente[screen][layer][i].igual(r, g, b))
			ret = i;
	}
	return ret;
}

u8 registraColor(u8 screen, u8 layer, u8 r, u8 g, u8 b){
	u8 usados = coloresFuenteUsados[screen][layer];
	if (usados < 16){
		coloresFuenteUsados[screen][layer]++;
	}else{
		usados = 15;
	}
	coloresFuente[screen][layer][usados].setColor(r, g, b);
	NF_DefineTextColor(screen, layer, usados, r, g, b);
	return usados;
}

void borraColores(u8 screen, u8 layer){
	for(u8 i=0;i<coloresFuenteUsados[screen][layer];i++){
		coloresFuente[screen][layer][i].definido = false;
		coloresFuente[screen][layer][i].red = 31;
		coloresFuente[screen][layer][i].green = 31;
		coloresFuente[screen][layer][i].blue = 31;
	}
	coloresFuenteUsados[screen][layer] = 0;
}

#define PJ_CHARS_NORM 100
#define PJ_CHARS_GRANDE 189
#define PJ_TOTAL_CHARS 289
#define PJ_TOTAL_CHARS_BLANCOS 329


void PJ_GirarFont(char *caracter, u8 rotation){
	int i,j;
	char copia[64];
	memcpy(copia,caracter,64);
	if (rotation == 1){ // Rotacion 90 a la derecha
		for(i=0;i<8;i++){
			for(j=0;j<8;j++){
				caracter[j*8+7-i] = copia[i*8+j];
			}
		}
	}else{				// Rotacion 90 a la izquierda (rotation = 2)
		for(i=0;i<8;i++){
			for(j=0;j<8;j++){
				caracter[(7-j)*8+i] = copia[i*8+j];
			}
		}
	}
	//free(copia);
}

void PJ_LoadTextFont(const char* file, const char* name, u16 width, u16 height, u8 rotation) {

	// Busca un slot libre
	u8 n = 0;
	u8 slot = 255;
	for (n = 0; n < NF_SLOTS_TBG; n ++) {		// Busca en todos los slots
		if (NF_TILEDBG[n].available) {			// Si esta libre
			NF_TILEDBG[n].available = false;	// Marcalo como en uso
			slot = n;							// Guarda el slot a usar
			n = NF_SLOTS_TBG;					// Deja de buscar
		}
	}
	// Si no hay ningun slot libre, error
	if (slot == 255) {
		NF_Error(103, "Tiled Bg", NF_SLOTS_TBG);
	}

	// Verifica que el fondo sea multiplo de 256px (32 tiles)
	u16 bx = width - ((width >> 8) << 8);
	u16 by = height - ((height >> 8) << 8);
	if ((bx > 0) || (by > 0)) {
		NF_Error(115, file, 0);
	}

	// Vacia los buffers que se usaran
	free(NF_BUFFER_BGMAP[slot]);		// Buffer para los mapas
	NF_BUFFER_BGMAP[slot] = NULL;
	free(NF_BUFFER_BGTILES[slot]);		// Buffer para los tiles
	NF_BUFFER_BGTILES[slot] = NULL;
	free(NF_BUFFER_BGPAL[slot]);		// Buffer para los paletas
	NF_BUFFER_BGPAL[slot] = NULL;

	// Declara los punteros a los ficheros
	FILE* file_id;

	// Variable para almacenar el path al archivo
	char filename[256];

	// Carga el archivo .FNT
	sprintf(filename, "%s/%s.fnt", NF_ROOTFOLDER, file);
	file_id = fopen(filename, "rb");
	if (file_id) {	// Si el archivo existe...
		// Obten el tamao del archivo
		NF_TILEDBG[slot].tilesize = (PJ_TOTAL_CHARS_BLANCOS << 6);		// 100 caracteres x 64 bytes
		// Reserva el espacio en RAM
		NF_BUFFER_BGTILES[slot] = (char*) calloc (NF_TILEDBG[slot].tilesize, sizeof(char));
		if (NF_BUFFER_BGTILES[slot] == NULL) {		// Si no hay suficiente RAM libre
			NF_Error(102, NULL, NF_TILEDBG[slot].tilesize);
		}
		// Lee el archivo y ponlo en la RAM
		fread (NF_BUFFER_BGTILES[slot], 1, 64, file_id);	// Lee el primer caracter (64 bytes)
		// ((((NF_TEXT_FONT_CHARS - 1) * 64) * rotation) + 64)
		if (rotation == 0){
			fread ((NF_BUFFER_BGTILES[slot] + 64), 1, (NF_TILEDBG[slot].tilesize - 64), file_id);	// Lee el resto de caracteres de la fuente
		}else{
			char caracter[64];
			for (int i=1;i<PJ_TOTAL_CHARS_BLANCOS;i++){
				fseek(file_id, i*64, SEEK_SET);
				fread (caracter, 1, 64, file_id);
				PJ_GirarFont(caracter,rotation);
				memcpy(NF_BUFFER_BGTILES[slot]+ i*64,caracter,64);
			}
		}
		//////fseek(file_id, ((((PJ_TOTAL_CHARS_BLANCOS - 1) << 6) * rotation) + 64), SEEK_SET);			// Y posicionate en esa posicion de archivo
		//////fread ((NF_BUFFER_BGTILES[slot] + 64), 1, (NF_TILEDBG[slot].tilesize - 64), file_id);	// Lee el resto de caracteres de la fuente
	} else {	// Si el archivo no existe...
		NF_Error(101, filename, 0);
	}
	fclose(file_id);		// Cierra el archivo
	// swiWaitForVBlank();		// Espera al cierre del archivo (Usar en caso de corrupcion de datos)

	// Crea un archivo .MAP vacio en RAM 
	// ((ancho / 8) * (alto / 8)) * 2
	NF_TILEDBG[slot].mapsize = (((width >> 3) * (height >> 3)) << 1);
	// Reserva el espacio en RAM
	NF_BUFFER_BGMAP[slot] = (char*) calloc (NF_TILEDBG[slot].mapsize, sizeof(char));
	if (NF_BUFFER_BGMAP[slot] == NULL) {		// Si no hay suficiente RAM libre
		NF_Error(102, NULL, NF_TILEDBG[slot].mapsize);
	}
	// Y ponlo a 0
	memset(NF_BUFFER_BGMAP[slot], 0, NF_TILEDBG[slot].mapsize);

	// Carga el archivo .PAL
	sprintf(filename, "%s/%s.pal", NF_ROOTFOLDER, file);
	file_id = fopen(filename, "rb");
	if (file_id) {	// Si el archivo existe...
		// Obten el tamao del archivo
		fseek(file_id, 0, SEEK_END);
		NF_TILEDBG[slot].palsize = ftell(file_id);
		rewind(file_id);
		// Reserva el espacio en RAM
		NF_BUFFER_BGPAL[slot] = (char*) calloc (NF_TILEDBG[slot].palsize, sizeof(char));
		if (NF_BUFFER_BGPAL[slot] == NULL) {		// Si no hay suficiente RAM libre
			NF_Error(102, NULL, NF_TILEDBG[slot].palsize);
		}
		// Lee el archivo y ponlo en la RAM
		fread (NF_BUFFER_BGPAL[slot], 1, NF_TILEDBG[slot].palsize, file_id);
	} else {	// Si el archivo no existe...
		NF_Error(101, filename, 0);
	}
	fclose(file_id);		// Cierra el archivo
	// swiWaitForVBlank();		// Espera al cierre del archivo (Usar en caso de corrupcion de datos)

	// Guarda el nombre del Fondo
	sprintf(NF_TILEDBG[slot].name, "%s", name);

	// Y las medidas
	NF_TILEDBG[slot].width = width;
	NF_TILEDBG[slot].height = height;

}

void PJ_getTilesCaracter(u8 caracter, int *tileUp, int* tileDown){

	u8 modificador = 0;
	bool calculado = false;
	int fila;

	if (caracter <= 0){
		*tileUp = 0;
		*tileDown = 0;
		calculado = true;
	}
	if (caracter == 2){ //Caracter "
		*tileUp = 320;
		*tileDown = 0;
		calculado = true;
	}
	if ((caracter > 2) && (caracter < 7))
		modificador = 1;
	if (caracter == 7){	//Caracter '
		*tileUp = 322;
		*tileDown = 0;
		calculado = true;
	}
	if ((caracter > 7) && (caracter < 10))
		modificador = 2;
	if (caracter == 10){//Caracter *
		*tileUp = 325;
		*tileDown = 0;
		calculado = true;
	}
	if (caracter == 11)
		modificador = 3;
	if (caracter == 12){//Caracter ,
		*tileUp = 0;
		*tileDown = 327;
		calculado = true;
	}
	if (caracter == 13){//Caracter -
		*tileUp = 321;
		*tileDown = 0;
		calculado = true;
	}
	if (caracter == 14){//Caracter .
		*tileUp = 0;
		*tileDown = 326;
		calculado = true;
	}
	if ((caracter > 14) && (caracter < 62))
		modificador = 6;
	if (caracter == 62){//Caracter ^
		*tileUp = 323;
		*tileDown = 0;
		calculado = true;
	}
	if (caracter == 63){//Caracter _
		*tileUp = 0;
		*tileDown = 328;
		calculado = true;
	}
	if (caracter == 64){//Caracter `
		*tileUp = 324;
		*tileDown = 0;
		calculado = true;
	}
	if (caracter > 64)
		modificador = 9;

	if (!calculado){
		fila = floor((caracter - modificador - 1) / 32);
		*tileUp = PJ_CHARS_NORM + caracter - modificador - 1 + 28 + fila * 32;
		*tileDown = PJ_CHARS_NORM + caracter - modificador + 31 + 28 + fila * 32;
	}
}

void PJ_WriteText(u8 screen, u8 layer, u8 x, u8 y, const char* text) {
	
	u8 r,g,b;
	int tileUp, tileDown;
	bool fuenteGrande = false;
	bool algoGrande = false;
	bool codigoControl = false;
	u8 colorF = 0;
	pilaU8 colores;
	colores.add(0);
	

	// Verifica si la capa de texto de destino existe
	if (!NF_TEXT[screen][layer].exist) {
		NF_Error(114, NULL, screen);
	}

	u16 n = 0;					// Variable de uso general

	u16 tsize = 0;				// Almacena el numero de letras
	tsize = strlen(text);		// Calcula el numero de letras del texto

	u8* string;					// Buffer temporal
	string = NULL;
	string = (u8*) calloc (tsize, sizeof(u8));

	// Almacena en el buffer temporal el valor de los caracteres
	for (n = 0; n < tsize; n ++) {
		string[n] = ((int)(text[n])) - 32;	// Resta 32 al valor entrado
		// Si es un caracter no valido, es un espacio
		if ((string[n] < 0) || (string[n] > 93)) string[n] = 0;
		// Verifica caracteres especiales
		if (((int)(text[n])) == 161) string[n] = 94;	// 
		if (((int)(text[n])) == 191) string[n] = 95;	// ?
		if (((int)(text[n])) == 199) string[n] = 96;	// 
		if (((int)(text[n])) == 231) string[n] = 97;	// 
		if (((int)(text[n])) == 209) string[n] = 98;	// 
		if (((int)(text[n])) == 241) string[n] = 99;	// 
		if (((int)(text[n])) == 10) string[n] = 200;	// \n
	}

	// Variable para calcular la posicion del texto
	s8 tx = 0;		// X
	s8 ty = 0;		// Y
	NF_SetTextColor(screen,layer,0);
	// Escribe los datos en el buffer de texto, segun la rotacion
	switch (NF_TEXT[screen][layer].rotation) {

		case 0:		// Sin rotacion
			// Traspasa las coordenadas virtuales a las reales
			tx = x;
			ty = y;
			// Copia el texto al buffer letra a letra
			for (n = 0; n < tsize; n ++) {
				codigoControl = false;
				//Si el caracter es "[" puede que estemos en una instruccion de control
				if (string[n] == 59){
					//Si el caracter siguiente es "/" es decir tenemos algo como "[/" puede que estemos cerrando
					//una instruccion de control.
					if (string[n+1] == 15){
						//si el texto a continuacion es "c]" es decir en total tenemos "[/c]" cierra control color
						if ((string[n+2] == 67) && (string[n+3] == 61)){
							//colores--;
							//if (colores < 0)
							//	colores = 0;
							//NF_SetTextColor(screen,layer,colores);
							colores.deleteUltimo();
							colorF = colores.getUltimo();
							NF_SetTextColor(screen,layer,colorF);
							n = n + 3;
							codigoControl = true;
						}
						//Si el texto a continuacion es de la forma "grande]" es decir en total tenemos "[/grande]"
						if ((string[n+2] == 71) && (string[n+3] == 82) && (string[n+4] == 65) && (string[n+5] == 78) && (string[n+6] == 68) && (string[n+7] == 69) && (string[n+8] == 61)){
							fuenteGrande = false;
							n = n + 8;
							codigoControl = true;
						}
					}else{
						//si el texto a continuacion es de la forma "c=??????]" es decir en total tenemos "[c=??????]"
						if ((string[n+1] == 67) && (string[n+2] == 29) && (string[n+9] == 61)){
							//Comprobamos que sean numeros lo que hay en los "?" de arriba
							if ((string[n+3] > 15) && (string[n+3] < 26)&&(string[n+4] > 15) && (string[n+4] < 26)&&(string[n+5] > 15) && (string[n+5] < 26) &&(string[n+6] > 15) && (string[n+6] < 26) &&(string[n+7] > 15) && (string[n+7] < 26) &&(string[n+8] > 15) && (string[n+8] < 26)){
								r = (string[n+3]-16) * 10 + string[n+4]-16;
								g = (string[n+5]-16) * 10 + string[n+6]-16;
								b = (string[n+7]-16) * 10 + string[n+8]-16;
								colorF = estaColor(screen,layer,r,g,b);
								if (colorF == 255){
									colorF = registraColor(screen, layer, r, g, b);
								}
								colores.add(colorF);
								NF_SetTextColor(screen,layer,colorF);
								n = n + 9;
								codigoControl = true;
							}
						}
						//Si el texto a continuacion es de la forma "grande]" es decir en total tenemos "[grande]"
						if ((string[n+1] == 71) && (string[n+2] == 82) && (string[n+3] == 65) && (string[n+4] == 78) && (string[n+5] == 68) && (string[n+6] == 69) && (string[n+7] == 61)){
							fuenteGrande = true;
							n = n + 7;
							codigoControl = true;
						}
					}
				}
				if (!codigoControl){
					// Si es un caracter valido
					if (string[n] < 100) {
						if (fuenteGrande){
							// Escribe la letra correspondiente usando un dos TILES
							PJ_getTilesCaracter(string[n],&tileUp,&tileDown);
							NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + tileUp));
							NF_SetTileOfMap(screen,layer, tx, ty+1, ((NF_TEXT[screen][layer].pal << 12) + tileDown));
							algoGrande = true;
						}else{
							// Escribe la letra correspondiente usando un unico TILE
							NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + string[n]));
						}
						// Siguiente letra
						tx ++;
					}
					if ((tx > NF_TEXT[screen][layer].width) || (string[n] == 200)) {		// Si llegas al final de linea,
						tx = 0;			// salto de linea
						ty ++;
						if (algoGrande)
							ty ++;
						algoGrande = false;
						if (ty > NF_TEXT[screen][layer].height) {	// Si estas en la ultima linea,
							ty = 0;		// vuelve a la primera
						}
					}
				}
			}
			break;


		case 1:		// Rotacion 90 a la derecha
			// Traspasa las coordenadas virtuales a las reales
			tx = (NF_TEXT[screen][layer].width - y);
			ty = x;
			// Copia el texto al buffer letra a letra
			for (n = 0; n < tsize; n ++) {
				//Si el caracter es "[" puede que estemos en una instruccion de control
				if (string[n] == 59){
					//Si el caracter siguiente es "/" es decir tenemos algo como "[/" puede que estemos cerrando
					//una instruccion de control.
					if (string[n+1] == 15){
						//si el texto a continuacion es "c]" es decir en total tenemos "[/c]" cierra control color
						if ((string[n+2] == 67) && (string[n+3] == 61)){
							//colores--;
							//if (colores < 0)
							//	colores = 0;
							//NF_SetTextColor(screen,layer,colores);
							colores.deleteUltimo();
							colorF = colores.getUltimo();
							NF_SetTextColor(screen,layer,colorF);
							n = n + 3;
							codigoControl = true;
						}
						//Si el texto a continuacion es de la forma "grande]" es decir en total tenemos "[/grande]"
						if ((string[n+2] == 71) && (string[n+3] == 82) && (string[n+4] == 65) && (string[n+5] == 78) && (string[n+6] == 68) && (string[n+7] == 69) && (string[n+8] == 61)){
							fuenteGrande = false;
							n = n + 8;
							codigoControl = true;
						}
					}else{
						//si el texto a continuacion es de la forma "c=??????]" es decir en total tenemos "[c=??????]"
						if ((string[n+1] == 67) && (string[n+2] == 29) && (string[n+9] == 61)){
							//Comprobamos que sean numeros lo que hay en los "?" de arriba
							if ((string[n+3] > 15) && (string[n+3] < 26)&&(string[n+4] > 15) && (string[n+4] < 26)&&(string[n+5] > 15) && (string[n+5] < 26) &&(string[n+6] > 15) && (string[n+6] < 26) &&(string[n+7] > 15) && (string[n+7] < 26) &&(string[n+8] > 15) && (string[n+8] < 26)){
								r = string[n+3] * 10 + string[n+4];
								g = string[n+5] * 10 + string[n+6];
								b = string[n+7] * 10 + string[n+8];
								colorF = estaColor(screen,layer,r,g,b);
								if (colorF == 255){
									colorF = registraColor(screen, layer, r, g, b);
								}
								colores.add(colorF);
								NF_SetTextColor(screen,layer,colorF);
								n = n + 9;
								codigoControl = true;
							}
						}
						//Si el texto a continuacion es de la forma "grande]" es decir en total tenemos "[grande]"
						if ((string[n+1] == 71) && (string[n+2] == 82) && (string[n+3] == 65) && (string[n+4] == 78) && (string[n+5] == 68) && (string[n+6] == 69) && (string[n+7] == 61)){
							fuenteGrande = true;
							n = n + 7;
							codigoControl = true;
						}
					}
				}
				if (!codigoControl){
					// Si es un caracter valido
					if (string[n] < 100) {
						if (fuenteGrande){
							// Escribe la letra correspondiente usando un dos TILES
							PJ_getTilesCaracter(string[n],&tileUp,&tileDown);
							NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + tileUp));
							NF_SetTileOfMap(screen,layer, tx-1, ty, ((NF_TEXT[screen][layer].pal << 12) + tileDown));
							algoGrande = true;
						}else{
							// Escribe la letra correspondiente usando un unico TILE
							NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + string[n]));
						}
						// Siguiente letra
						ty ++;
					}
					if ((ty > NF_TEXT[screen][layer].height) || (string[n] == 200)) {		// Si llegas al final de linea,
						ty = 0;			// salto de linea
						tx --;
						if (algoGrande)
							tx --;
						algoGrande = false;
						if (tx < 0) {	// Si estas en la ultima linea,
							tx = NF_TEXT[screen][layer].width;	// vuelve a la primera
						}
					}
				}
			}
			break;


		case 2:		// Rotacion 90 a la izquierda
			// Traspasa las coordenadas virtuales a las reales
			tx = y;
			ty = (NF_TEXT[screen][layer].height - x);
			// Copia el texto al buffer letra a letra
			for (n = 0; n < tsize; n ++) {
				//Si el caracter es "[" puede que estemos en una instruccion de control
				if (string[n] == 59){
					//Si el caracter siguiente es "/" es decir tenemos algo como "[/" puede que estemos cerrando
					//una instruccion de control.
					if (string[n+1] == 15){
						//si el texto a continuacion es "c]" es decir en total tenemos "[/c]" cierra control color
						if ((string[n+2] == 67) && (string[n+3] == 61)){
							//colores--;
							//if (colores < 0)
							//	colores = 0;
							//NF_SetTextColor(screen,layer,colores);
							colores.deleteUltimo();
							colorF = colores.getUltimo();
							NF_SetTextColor(screen,layer,colorF);
							n = n + 3;
							codigoControl = true;
						}
						//Si el texto a continuacion es de la forma "grande]" es decir en total tenemos "[/grande]"
						if ((string[n+2] == 71) && (string[n+3] == 82) && (string[n+4] == 65) && (string[n+5] == 78) && (string[n+6] == 68) && (string[n+7] == 69) && (string[n+8] == 61)){
							fuenteGrande = false;
							n = n + 8;
							codigoControl = true;
						}
					}else{
						//si el texto a continuacion es de la forma "c=??????]" es decir en total tenemos "[c=??????]"
						if ((string[n+1] == 67) && (string[n+2] == 29) && (string[n+9] == 61)){
							//Comprobamos que sean numeros lo que hay en los "?" de arriba
							if ((string[n+3] > 15) && (string[n+3] < 26)&&(string[n+4] > 15) && (string[n+4] < 26)&&(string[n+5] > 15) && (string[n+5] < 26) &&(string[n+6] > 15) && (string[n+6] < 26) &&(string[n+7] > 15) && (string[n+7] < 26) &&(string[n+8] > 15) && (string[n+8] < 26)){
								r = string[n+3] * 10 + string[n+4];
								g = string[n+5] * 10 + string[n+6];
								b = string[n+7] * 10 + string[n+8];
								colorF = estaColor(screen,layer,r,g,b);
								if (colorF == 255){
									colorF = registraColor(screen, layer, r, g, b);
								}
								colores.add(colorF);
								NF_SetTextColor(screen,layer,colorF);
								n = n + 9;
								codigoControl = true;
							}
						}
						//Si el texto a continuacion es de la forma "grande]" es decir en total tenemos "[grande]"
						if ((string[n+1] == 71) && (string[n+2] == 82) && (string[n+3] == 65) && (string[n+4] == 78) && (string[n+5] == 68) && (string[n+6] == 69) && (string[n+7] == 61)){
							fuenteGrande = true;
							n = n + 7;
							codigoControl = true;
						}
					}
				}
				if (!codigoControl){
					// Si es un caracter valido
					if (string[n] < 100) {
						if (fuenteGrande){
							// Escribe la letra correspondiente usando un dos TILES
							PJ_getTilesCaracter(string[n],&tileUp,&tileDown);
							NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + tileUp));
							NF_SetTileOfMap(screen,layer, tx+1, ty, ((NF_TEXT[screen][layer].pal << 12) + tileDown));
							algoGrande = true;
						}else{
							// Escribe la letra correspondiente usando un unico TILE
							NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + string[n]));
						}
						// Escribe la letra correspondiente
						//NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + string[n]));
						// Siguiente letra
						ty --;
					}
					if ((ty < 0) || (string[n] == 200)) {		// Si llegas al final de linea,
						ty = NF_TEXT[screen][layer].height;		// Salto de linea
						tx ++;
						if (algoGrande)
							tx ++;
						algoGrande = false;
						if (tx > NF_TEXT[screen][layer].width) {	// Si llegas a la ultima linea
							tx = 0;		// vuelve a la primera
						}
					}
				}
			}
			break;

	}

	// Marca esta capa de texto para actualizar
	NF_TEXT[screen][layer].update = true;

	// Libera el buffer
	free(string);

}


void PJ_EscribeConversacion(u8 screen, u8 layer, const char* text) {
	
	u8 r,g,b;
	int tileUp, tileDown;
	bool codigoControl = false;
	u8 colorF = 0;
	ctrConv.colores.add(0);
	//u8 nLeidos = 0;
	bool leido = false;
	u8 leidosControl = 0;
	

	// Verifica si la capa de texto de destino existe
	if (!NF_TEXT[screen][layer].exist) {
		NF_Error(114, NULL, screen);
	}

	u16 n = 0;					// Variable de uso general

	u16 tsize = 0;				// Almacena el numero de letras
	tsize = strlen(text);		// Calcula el numero de letras del texto

	u8* string;					// Buffer temporal
	string = NULL;
	string = (u8*) calloc (tsize, sizeof(u8));

	// Almacena en el buffer temporal el valor de los caracteres
	for (n = 0; n < tsize; n ++) {
		string[n] = ((int)(text[n])) - 32;	// Resta 32 al valor entrado
		// Si es un caracter no valido, es un espacio
		if ((string[n] < 0) || (string[n] > 93)) string[n] = 0;
		// Verifica caracteres especiales
		if (((int)(text[n])) == 161) string[n] = 94;	// 
		if (((int)(text[n])) == 191) string[n] = 95;	// ?
		if (((int)(text[n])) == 199) string[n] = 96;	// 
		if (((int)(text[n])) == 231) string[n] = 97;	// 
		if (((int)(text[n])) == 209) string[n] = 98;	// 
		if (((int)(text[n])) == 241) string[n] = 99;	// 
		if (((int)(text[n])) == 10) string[n] = 200;	// \n
	}

	// Variable para calcular la posicion del texto
	u8 tx = textX;		// X
	u8 ty = textY;		// Y
	//NF_SetTextColor(screen,layer,0);
	// Escribe los datos en el buffer de texto

	// Copia el texto al buffer letra a letra
	//for (n = caracteresLeidos; (n < tsize) && (nLeidos < velEscritura); n ++) {
	for (n = caracteresLeidos; (n < tsize) && !leido; n ++) {
		codigoControl = false;
		//Si el caracter es "[" puede que estemos en una instruccion de control
		if (string[n] == 59){
			//Si el caracter siguiente es "/" es decir tenemos algo como "[/" puede que estemos cerrando
			//una instruccion de control.
			if (string[n+1] == 15){
				//si el texto a continuacion es "c]" es decir en total tenemos "[/c]" cierra control color
				if ((string[n+2] == 67) && (string[n+3] == 61)){
					ctrConv.colores.deleteUltimo();
					colorF = ctrConv.colores.getUltimo();
					NF_SetTextColor(screen,layer,colorF);
					n = n + 3;
					codigoControl = true;
					leidosControl = leidosControl + 4;
				}
				//Si el texto a continuacion es de la forma "grande]" es decir en total tenemos "[/grande]"
				if ((string[n+2] == 71) && (string[n+3] == 82) && (string[n+4] == 65) && (string[n+5] == 78) && (string[n+6] == 68) && (string[n+7] == 69) && (string[n+8] == 61)){
					ctrConv.fuenteGrande = false;
					n = n + 8;
					codigoControl = true;
					leidosControl = leidosControl + 9;
				}
			}else{
				//si el texto a continuacion es de la forma "c=??????]" es decir en total tenemos "[c=??????]"
				if ((string[n+1] == 67) && (string[n+2] == 29) && (string[n+9] == 61)){
					//Comprobamos que sean numeros lo que hay en los "?" de arriba
					if ((string[n+3] > 15) && (string[n+3] < 26)&&(string[n+4] > 15) && (string[n+4] < 26)&&(string[n+5] > 15) && (string[n+5] < 26) &&(string[n+6] > 15) && (string[n+6] < 26) &&(string[n+7] > 15) && (string[n+7] < 26) &&(string[n+8] > 15) && (string[n+8] < 26)){
						r = (string[n+3]-16) * 10 + string[n+4]-16;
						g = (string[n+5]-16) * 10 + string[n+6]-16;
						b = (string[n+7]-16) * 10 + string[n+8]-16;
						colorF = estaColor(screen,layer,r,g,b);
						if (colorF == 255){
							colorF = registraColor(screen, layer, r, g, b);
						}
						ctrConv.colores.add(colorF);
						NF_SetTextColor(screen,layer,colorF);
						n = n + 9;
						codigoControl = true;
						leidosControl = leidosControl + 10;
					}
				}
				//Si el texto a continuacion es de la forma "grande]" es decir en total tenemos "[grande]"
				if ((string[n+1] == 71) && (string[n+2] == 82) && (string[n+3] == 65) && (string[n+4] == 78) && (string[n+5] == 68) && (string[n+6] == 69) && (string[n+7] == 61)){
					ctrConv.fuenteGrande = true;
					n = n + 7;
					codigoControl = true;
					leidosControl = leidosControl + 8;
				}
			}
		}
		if (!codigoControl){
			// Si es un caracter valido
			if (string[n] < 100) {
				//nLeidos++;
				leido = true;
				if (ctrConv.fuenteGrande){
					// Escribe la letra correspondiente usando un dos TILES
					PJ_getTilesCaracter(string[n],&tileUp,&tileDown);
					NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + tileUp));
					NF_SetTileOfMap(screen,layer, tx, ty+1, ((NF_TEXT[screen][layer].pal << 12) + tileDown));
					ctrConv.algoGrande = true;
				}else{
					// Escribe la letra correspondiente usando un unico TILE
					NF_SetTileOfMap(screen,layer, tx, ty, ((NF_TEXT[screen][layer].pal << 12) + string[n]));
				}
				// Siguiente letra
				tx ++;
			}
			if ((tx > NF_TEXT[screen][layer].width) || (string[n] == 200)) {		// Si llegas al final de linea,
				tx = 0;			// salto de linea
				ty ++;
				if (ctrConv.algoGrande)
					ty ++;
				ctrConv.algoGrande = false;
				if (ty > NF_TEXT[screen][layer].height) {	// Si estas en la ultima linea,
					ty = 0;		// vuelve a la primera
				}
			}
		}
	}

	textX = tx;
	textY = ty;
	// Marca esta capa de texto para actualizar
	NF_TEXT[screen][layer].update = true;

	// Libera el buffer
	free(string);
	//caracteresLeidos = caracteresLeidos + nLeidos + leidosControl;
	caracteresLeidos = caracteresLeidos + leidosControl;
	if (leido)
		caracteresLeidos++;

}